@echo off
ECHO ##################################################
ECHO ####  The very first start may take a minute  ####
ECHO ####  To use Jupyter please open web browser  ####
ECHO ####          http://localhost:8888/          ####
ECHO ##################################################
IF NOT EXIST init.json (
    ECHO {"jupyter": true} > init.json
    ECHO.
    ECHO Jupyter initialization will start in a few seconds
    ECHO.
    timeout 9
)
uv add jupyter
rem uv add jupyter_contrib_nbextensions
rem uv add jupyter-autocomplete
uv run --with jupyter jupyter lab
pause