@echo off
ECHO ##################################################
ECHO ####         Type following commands:         ####
ECHO ##################################################
ECHO.
ECHO from uurest import *
ECHO print(fetch_setup())
ECHO fetch("http://www.somenonexistingwebpage.net")
ECHO response = fetch("https://upload.wikimedia.org/wikipedia/commons/2/2f/Google_2015_logo.svg")
ECHO fetch_setup()["verbose_level"] = 1
ECHO response.save_text("./google_logo.svg")
ECHO response = fetch('https://fakestoreapi.com/products')
ECHO print(response)
ECHO response.json["itemList"][0]
ECHO.
uv run python